==> kite-1.0b3.tar.gz: kite-1.0b3/modules/__internal/_exception.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <pthread.h>
#include "kite_object.h"
#include "vm/kite_vm.h"

/*****************************************************************************
 * Class constructor
 ****************************************************************************/
KITE_CLASS_METHOD(Exception_construct) {
    kite_set_property(thd, this, "msg", 
        kite_reference_object(args->builtin_data.listvalue.car));
}

/*****************************************************************************
 * String representation of object
 ****************************************************************************/
KITE_CLASS_METHOD(Exception_str) {
    KITE_NO_ARGS;
    kite_vm_return(thd, kite_object_name(thd, this));
}

/*****************************************************************************
 * Throw given exception
 ****************************************************************************/
KITE_CLASS_METHOD(Exception_throw) {
    KITE_NO_ARGS;
    if (thd->exception) {
        kite_dereference_object(thd->exception);
    }
    thd->exception = kite_reference_object(this);
    
    /* Build stack trace */
    kite_object_t *trace = NULL, *curtrace = NULL;
    kite_stack_entry_t *funcstack =
        &thd->func_stack->stack[thd->func_stack->length - 2];
    int i = thd->func_stack->length - 2;
    while(i >= 0) {
        char buf[1024];
        kite_funccall_info_t *s = 
            (kite_funccall_info_t*)funcstack->obj;
        if (!trace) {
            trace = kite_new_list(thd);
            curtrace = trace;
        }
        if (s->line > 0) {
            sprintf(buf, "called from %s line %d", s->file, s->line);
        } else {
            sprintf(buf, "called from %s", s->file);
        }
        curtrace->builtin_data.listvalue.car = kite_new_string(thd, buf);
        if (i > 0) {
            curtrace->builtin_data.listvalue.cdr = kite_new_list(thd);
        }
        curtrace = curtrace->builtin_data.listvalue.cdr;
        funcstack = &thd->func_stack->stack[--i];
    }
    if (!trace) trace = kite_new_list(thd);
    kite_set_property(thd, this, "trace", trace);
}

KITE_MODULE_INITIALIZER(_exception) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits =
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = kite_new_class(thread, inherits, "_exception");
    
    /* Add methods common to all classes. */
    kite_add_method(thread, newclass, "__construct__",
        kite_new_method(thread, FUNC_COMPILED, Exception_construct, 1));
    kite_add_method(thread, newclass, "str", 
        kite_new_method(thread, FUNC_COMPILED, Exception_str, 0));
    kite_add_method(thread, newclass, "throw", 
        kite_new_method(thread, FUNC_COMPILED, Exception_throw, 0));
    
    /* Add common exception properties. */
    kite_add_property(thread, newclass, "msg", 0);
    kite_add_property(thread, newclass, "trace", 0);
    
    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "_exception", 1);
    kite_set_property(thread, parent, "_exception", newclass);
}

==> kite_1.0.2.orig.tar.gz: kite-1.0.2/modules/__internal/_exception.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include "kite_object.h"
#include "vm/kite_vm.h"

#ifndef WIN32
#include <pthread.h>
#endif /* WIN32 */

/*****************************************************************************
 * Class constructor
 ****************************************************************************/
KITE_CLASS_METHOD(Exception_construct) {
    kite_set_property(thd, this, "msg", 
        kite_reference_object(args->builtin_data.listvalue.car));
}

/*****************************************************************************
 * String representation of object
 ****************************************************************************/
KITE_CLASS_METHOD(Exception_str) {
    KITE_NO_ARGS;
    kite_vm_return(thd, kite_object_name(thd, this));
}
/*****************************************************************************
 * Perror of errno
 ****************************************************************************/
KITE_CLASS_METHOD(Exception_perror) {
    char str[256];

    (void)this;
    KITE_NO_ARGS;

    perror(str);
    kite_vm_return(thd, kite_new_string(thd, str));
}

/*****************************************************************************
 * Throw given exception
 ****************************************************************************/
KITE_CLASS_METHOD(Exception_throw) {
    kite_object_t *trace = NULL, *curtrace = NULL;
    kite_stack_entry_t *funcstack;
    int i;

    KITE_NO_ARGS;
    if (thd->exception) {
        kite_dereference_object(thd->exception);
    }
    thd->exception = kite_reference_object(this);
    
    /* Build stack trace */
    funcstack =
        &thd->func_stack->stack[thd->func_stack->length - 2];
    i = thd->func_stack->length - 2;
    while(i >= 0) {
        char buf[1024];
        kite_funccall_info_t *s = 
            (kite_funccall_info_t*)funcstack->obj;
        if (!trace) {
            trace = kite_new_list(thd);
            curtrace = trace;
        }
        if (s->line > 0) {
            sprintf(buf, "called from %s line %d", s->file, s->line);
        } else {
            sprintf(buf, "called from %s", s->file);
        }
        curtrace->builtin_data.listvalue.car = kite_new_string(thd, buf);
        if (i > 0) {
            curtrace->builtin_data.listvalue.cdr = kite_new_list(thd);
        }
        curtrace = curtrace->builtin_data.listvalue.cdr;
        funcstack = &thd->func_stack->stack[--i];
    }
    if (!trace) trace = kite_new_list(thd);
    kite_set_property(thd, this, "trace", trace);
}

KITE_MODULE_INITIALIZER(_exception) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits =
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = kite_new_class(thread, inherits, "_exception");
    kite_set_docstring(newclass, "Exception object internal code.");
    
    /* Add methods common to all classes. */
    kite_add_method(thread, newclass, "__construct__",
        kite_new_method_compiled_with_docs(thread, Exception_construct, 
                                  "Constructs new exception object.", 1,
                                  "msg", "The exception's message."));
    kite_add_method(thread, newclass, "str", 
        kite_new_method_compiled_with_docs(thread, Exception_str, 
                                  "Returns string representation of exception.", 0));
    kite_add_method(thread, newclass, "get_perror", 
        kite_new_method_compiled_with_docs(thread, Exception_perror, 
                                  "Internal use only.", 0));
    kite_add_method(thread, newclass, "throw", 
        kite_new_method_compiled_with_docs(thread, Exception_throw, 
                                  "Throws exception to VM for handling.", 0));
    
    /* Add common exception properties. */
    kite_add_property(thread, newclass, "msg", 0, "Message passed to exception.");
    kite_add_property(thread, newclass, "trace", 0, "Exception stack trace.");
    
    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "_exception", 1, NULL);
    kite_set_property(thread, parent, "_exception", newclass);
}
